<?php

/**
 * Helper to determine if the current user can administer the marketplace, meaning they have either the
 * marketplace_administrator role or administrator role.
 *
 * @param int|null $user_id
 *
 * @return bool
 */
function bt_can_admin_marketplace( $user_id = null ) {
	if ( empty( $user_id ) ) {
		$user_id = get_current_user_id();
	}

	return wc_user_has_role( $user_id, 'administrator' )
	       || wc_user_has_role( $user_id, 'marketplace_administrator' );
}


/**
 * Filter the admin footer text to customize.
 * If site uses Sellers, it will have a "Powered by VenYou Marketplace" line.
 * Else it will have a "Need help? Contact Bicycle Theory" line.
 */
add_filter( 'admin_footer_text', function( $text ) {
	//error_log( 'ADMIN FOOTER: text = ' . $text );

	$text = '<span id="footer-thankyou"> ';
	if ( bt_use_sellers() ) {
		$text .= get_option( 'blogname' ) . ' is powered by '
		         . '<a href="https://venyou.com/marketplace/" target="_blank">VenYou Marketplace</a>.';
	} else {
		$text .= 'Need help? Contact '
		         . '<a href="https://bicycletheory.com/contact/" target="_blank">Bicycle Theory</a>.';
	}
	$text .= '</span>';

	return $text;
}, 9988, 1 );


/**
 * Remove the WP version/update from footer.
 */
// NOT WORKING remove_filter( 'update_footer', 'core_update_footer', 10 );


/**
 * Filter the WP Login redirect, so that we can send Sellers directly to the list of orders.
 */
add_filter( 'login_redirect', function ( $redirect_to, $request, $user ) {
	//error_log( print_r( $redirect_to, 1) );
	//error_log( print_r( $request, 1) );
	//error_log( print_r( $user, 1) );

	// send sellers to the list of orders
	if ( bt_is_seller( $user ) ) {
		//error_log( '[login_redirect]  RETURNING CUSTOM URL' );
		return '/wp-admin/edit.php?post_type=shop_order';
	}

	// default / as it was
	return $redirect_to;
}, 1, 3 );


/**
 * Filter queries in admin when we have sellers and the user is a seller, so those users can only see their posts
 */
add_filter('parse_query', function ( $wp_query ) {
	// get the post type used for sellers, if any, in part to determine if we are using sellers
	$seller_post_type = bt_use_sellers( 'seller_post_type' );

	// @todo is there a woocommerce query parallel we need to intercept? how do we filter the counts for status links/filters?
	//error_log( ' ===== ADMIN PARSE QUERY: ' . print_r( $wp_query, 1 ) );

	// actions to take if we use sellers and current user is a non-admin seller
	if ( ! empty( $seller_post_type ) // could be array
	     && (
	     	bt_is_seller()
			|| bt_is_subscriber_could_be_seller()
	     )
	     && is_admin()
	) {
		// make sure we're dealing with post type as an array
		if ( is_string( $seller_post_type ) ) {
			$seller_post_type = [$seller_post_type];
		}
		// if ajax, check what for, because we need to filter media library (grid) but we can't mess with add to cart
		if ( wp_doing_ajax() && in_array( $wp_query->get( 'post_type' ), $seller_post_type ) ) {
			// when doing ajax query, if it's for the seller post type, then it's an add-to-cart call where seller info
			// is also being retrieved. so, we just return without modifying the query.
			//error_log( ' ===== AJAX PARSE QUERY: ' . print_r( $wp_query, 1 ) );
			return;
		}

		// determine if we're going to filter the query for author
		$filter_for_author = false;

		// check for general post edit screen - this catches completed/processing orders when the query doesn't!
		if ( strpos( $_SERVER[ 'REQUEST_URI' ], '/wp-admin/edit.php' ) !== false ) {
			$filter_for_author = true;
		}

		if ( ! $filter_for_author ) {
			// define which post types need to be author-only
			$filter_post_types = [
				'any',
				'post',
				'product',
				'wc_order',
				'shop_coupon',
				'shop_subscription',
				'attachment',
			];
			// include seller post type(s)
			$filter_post_types = array_merge( $filter_post_types, $seller_post_type );
			// if we're querying for one of those author-only post types, add author arg to query
			if ( in_array( $wp_query->get( 'post_type' ), $filter_post_types ) ) {
				$filter_for_author = true;
			}
		}

		if ( $filter_for_author ) {
			global $current_user;
			$wp_query->set( 'author', $current_user->ID );
		}
	}
}, 9988 );


/**
 * Adjust admin menu
 */
add_action( 'admin_menu', function() {
	/* to see what's in the menus: * /
	global $submenu;
	error_log( 'SUBMENU: ' . print_r( $submenu, 1 ) );
	// * */

	/**
	 * remove things for shop managers (sellers)
	 */
	if ( bt_is_seller() || bt_is_subscriber_could_be_seller() ) {
    // loop through menu items to rename WooCommerce (need to do this since WC 4.4)
    // this was previously removed but reintroduced 2024-12 because cml/dm still relied on this - menu is different from cml/nma for some reason, maybe subscriptions/roles
    foreach( $GLOBALS['menu'] as $key => $menu ) {
      if ( $menu[0] == 'WooCommerce' ) {
        $GLOBALS['menu'][$key][0] = 'Sales';
      }
    }

    // remove woocommerce main (except orders & report), analytics, and marketing
		// THIS CHANGED SINCE WC 4.4 remove_menu_page( 'woocommerce' );
		remove_menu_page( 'wc-admin&path=/analytics/overview' );
		remove_menu_page( 'wc-admin&path=/analytics/revenue' );
		remove_menu_page( 'wc-admin&path=/customers' );
		remove_menu_page( 'wc-admin&path=/marketing' );
		remove_menu_page( 'woocommerce-marketing' );
		remove_submenu_page( 'woocommerce', 'wc-admin&path=/customers' );
		remove_submenu_page( 'woocommerce', 'wc-admin&path=/extensions' );
		remove_menu_page( 'edit.php?post_type=shop_subscription' );
		//remove_submenu_page( 'woocommerce', 'wc-reports' );
		remove_submenu_page( 'woocommerce', 'wc-settings' );
		remove_submenu_page( 'woocommerce', 'wc-status' );
		remove_submenu_page( 'woocommerce', 'wc-addons' );
		remove_submenu_page( 'woocommerce', 'wc-admin' );
		remove_submenu_page( 'woocommerce', 'wc-order-export' );
		remove_submenu_page( 'edit.php?post_type=product', 'edit-tags.php?taxonomy=product_cat&amp;post_type=product' );
		remove_submenu_page( 'edit.php?post_type=product', 'edit-tags.php?taxonomy=product_tag&amp;post_type=product' );
		remove_submenu_page( 'edit.php?post_type=product', 'product_attributes' );
		remove_submenu_page( 'edit.php?post_type=product', 'product-reviews' );
    remove_submenu_page( 'edit.php?post_type=shop_order', 'post-new.php?post_type=shop_order' );

		// remove searchwp stats, within dashboard/home
		remove_submenu_page( 'index.php', 'searchwp-stats' );

		// remove tiny compress images stats, within media
		remove_submenu_page( 'upload.php', 'tiny-bulk-optimization' );

		// remove order export
		remove_menu_page( 'wc-order-export' );

		// remove automate woo
		remove_menu_page( 'automatewoo' );

		// remove add new CPT for seller
		$seller_post_type = bt_use_sellers( 'seller_post_type' );
		if ( is_array( $seller_post_type ) ) {
			// determine the seller type out of the multiple
			$seller_post = bt_get_seller_post( get_current_user_id() );

			// remove things for each post type
			foreach( $seller_post_type as $post_type ) {
				// remove the add new for everything
				remove_submenu_page( 'edit.php?post_type='.$post_type, 'post-new.php?post_type='.$post_type );

				// remove the whole post type for anything but this seller's post type
				if ( $post_type != $seller_post->post_type ) {
					//error_log( 'ADMIN MENU: loop post type = ' . $post_type . ' / seller post type = ' . $seller_post->post_type );
					remove_menu_page( 'edit.php?post_type='.$post_type );
				}
			}

		} else {
			remove_submenu_page( 'edit.php?post_type='.$seller_post_type, 'post-new.php?post_type='.$seller_post_type );
		}

		// remove tools
		remove_menu_page( 'tools.php' );

		// remove ACF options within settings (will be disallowed if attempted)
		remove_submenu_page( 'options-general.php', 'bt-options' );

		// remove profile which we will replace
		remove_menu_page( 'profile.php' );
		add_menu_page(
			'User Account',
			'User Account',
			'read',
			'profile.php'
		);
    // remove user tags
    remove_submenu_page( 'profile.php', 'edit-tags.php?taxonomy=user_tag' );

	}

  // remove the WC payments menu
  remove_menu_page( 'admin.php?page=wc-admin&task=payments' );

}, 9988 );


/**
 * Adjust admin bar (top) menu
 */
add_action( 'admin_bar_menu', function( $wp_admin_bar ) {
	/*
	to see what's in the menus:
	- inspect the node in a browser
	- find the ID like "wp-admin-bar-new-content"
	- remove with slug "new-content" in the above example
	*/

	// actions for sellers
	if ( bt_is_seller() || bt_is_subscriber_could_be_seller() ) {
		// post object representing the seller
		$seller_post = bt_get_seller_post( get_current_user_id() );
		//error_log( 'ADMIN BAR: seller post = ' . print_r( $seller_post, 1 ) );

		// remove standard dashboard. maybe someday we'll have something useful there.
		$wp_admin_bar->remove_node( 'dashboard' );

		// remove the woocommerce shop page link and replace with seller page
		$seller_page_name = bt_use_sellers( 'seller_page_name' )
			? bt_use_sellers( 'seller_page_name' )
			: 'Shop';
		$title_html = 'View My ' . bt_get_post_type_name( $seller_post ) . ' ' . $seller_page_name;
		$wp_admin_bar->remove_node( 'view-store' );
		$wp_admin_bar->add_node( [
			'parent' => 'site-name', // within the home/dashboard item
			'id' => 'bt-seller-store',
			'title' => $title_html,
			'href'  => get_the_permalink( $seller_post ),
			'meta'  => [
				'class' => 'seller-store',
			],
		] );

		// add link to front end my-account
		$account_marketplace_name = bt_use_sellers( 'marketplace_name' )
			? bt_use_sellers( 'marketplace_name' )
			: 'Curbside';
		$wp_admin_bar->add_node( [
			'parent' => 'site-name', // within the home/dashboard item
			'id' => 'bt-seller-my-account',
			'title' => 'My ' . $account_marketplace_name . ' Account',
			'href'  => bt_my_account_url(),
			'meta'  => [
				'class' => 'seller-my-account',
			],
		] );

		// remove the "+ New" menu
		$wp_admin_bar->remove_node( 'new-content' );

		// remove the "View" current post button
		$wp_admin_bar->remove_node( 'view' );

		// remove the "Edit" current post button
		$wp_admin_bar->remove_node( 'edit' );

		// remove the "View X" archives button
		$wp_admin_bar->remove_node( 'archive' );

		// remove the "Edit Profile" link within the Howdy sub menu
		$wp_admin_bar->remove_node( 'edit-profile' );

		// manage seller info button
		$title_html = '<span class="icon"></span><span class="ab-label">Edit ' . bt_get_post_type_name( $seller_post ) . ' Info</span>';
		$wp_admin_bar->add_node( [
			'id' => 'bt-seller-info',
			'title' => $title_html,
			'href'  => get_admin_url() . 'post.php?post=' . $seller_post->ID . '&action=edit',
			'meta'  => [
				'class' => 'seller-info',
			],
		] );

		if ( bt_is_seller() ) {
			// manage orders button
			$title_html = 'Orders';
			if (
				$open_order_count = bt_get_user_post_count(
					get_current_user_id(),
					'shop_order',
					"'wc-processing','wc-ready-pickup'"
				)
			) {
				$title_html .= ' <span class="admin-bar-count">' . $open_order_count . '</span>';
			}
			$title_html = '<span class="icon"></span><span class="ab-label">' . $title_html . '</span>';
			$wp_admin_bar->add_node( [
				'id' => 'bt-orders',
				'title' => $title_html,
				'href'  => get_admin_url() . 'edit.php?post_type=shop_order',
				'meta'  => [
					'class' => 'orders',
				],
			] );

			// manage products button
			$title_html = '<span class="icon"></span><span class="ab-label">Products</span>';
			$wp_admin_bar->add_node( [
				'id' => 'bt-products',
				'title' => $title_html,
				'href'  => get_admin_url() . 'edit.php?post_type=product',
				'meta'  => [
					'class' => 'products',
				],
			] );

			// add custom New Product button, child of products
			$title_html = '<span class="icon"></span><span class="ab-label">Add New Product</span>';
			$wp_admin_bar->add_node( [
				'id' => 'bt-new-product',
				'parent' => 'bt-products',
				'title' => $title_html,
				'href'  => get_admin_url() . 'post-new.php?post_type=product',
				'meta'  => [
					'class' => 'new-product',
				],
			] );
		}

		// add custom Help button
		$title_html = '<span class="icon"></span><span class="ab-label">Help</span>';
		$wp_admin_bar->add_node( [
			'id' => 'bt-get-help',
			'title' => $title_html,
			'href'  => ( bt_use_sellers( 'admin_bar_help_url' )
				? bt_use_sellers( 'admin_bar_help_url' )
				: '/help/'
			),
			'meta'  => [
				'class' => 'get-help',
			],
		] );
	}

}, 9988 );


/**
 * Use the seller logo for avatar.
 */
add_action( 'get_avatar', function( $avatar, $id_or_email, $size, $default, $alt, $args ) {
	/*
	error_log ( "AVATAR FILTER: avatar = $avatar" );
	error_log ( "AVATAR FILTER: id_or_email = $id_or_email" );
	error_log ( "AVATAR FILTER: size = $size" );
	error_log ( "AVATAR FILTER: default = $default" );
	error_log ( "AVATAR FILTER: alt = $alt" );
	error_log ( "AVATAR FILTER: args = " . print_r( $args, 1 ) );
	*/

	global $current_user;

	if ( ( bt_is_seller() || bt_is_subscriber_could_be_seller() ) && $current_user->ID == $id_or_email ) {
		$seller_post = bt_get_seller_post();
		if ( $logo_image = get_field( 'logo_image', $seller_post ) ) {
			//error_log ( "AVATAR FILTER: logo = " . print_r( $logo_image, 1 ) );
			$avatar = '<img class="avatar avatar-64 photo" src="' . $logo_image['sizes']['thumbnail'] . '" alt="Logo image for ' . $seller_post->post_title . '">';
			//error_log ( "AVATAR FILTER: avatar ADJUSTED = $avatar" );
		}
	}

	return $avatar;
}, 10, 6 );


/**
 * Dashboard:
 * - Remove default WordPress and WooCommerce widgets.
 * - Add custom widgets.
 */
add_action( 'wp_dashboard_setup', function() {
	// wp events and news
	remove_meta_box( 'dashboard_primary', 'dashboard', 'side' );
	// quick draft
	remove_meta_box( 'dashboard_quick_press', 'dashboard', 'side' );
	// activity
	remove_meta_box( 'dashboard_activity', 'dashboard', 'normal');

	if ( bt_is_seller() || bt_is_subscriber_could_be_seller() ) {
		// remove Welcome panel
		remove_action( 'welcome_panel', 'wp_welcome_panel' );
		// remove the rest of the default WP dashboard widgets
		remove_meta_box( 'health_check_status', 'dashboard', 'normal' );
		remove_meta_box( 'dashboard_right_now', 'dashboard', 'normal' );
		// remove WooCommerce status widget
		remove_meta_box( 'woocommerce_dashboard_status', 'dashboard', 'normal');
		// remove tinypng dashboard widget
		remove_meta_box( 'tinypng_dashboard_widget', 'dashboard', 'normal');

		// add hello/welcome
		wp_add_dashboard_widget(
			'bt_dashboard_hello', // widget slug.
			esc_html__( 'Welcome to ' . get_bloginfo( 'name' ), 'bt' ), // title.
			'bt_dashboard_hello_render' // display function.
		);
	}
} );


/**
 * Dashboard widget render for Welcome.
 */
function bt_dashboard_hello_render() {
	if ( is_string( $content = bt_use_sellers( 'admin_dashboard_content' ) ) ) {
		_e( $content, 'bt' );
	} else {
		$content = '<p>Hello!</p>';
		//$content .= '<p>Soon we will have some useful dashboard widgets here.</p>';
		if ( $help_email = bt_use_sellers( 'help_email' ) ) {
			$content .= '<p>While we build out "help" content, if you have questions, email us:</p>';
			$content .= '<p><a href="mailto:' . $help_email . '">' . $help_email . '</a></p>';
		}
		_e( $content, 'bt' );
	}
}


/**
 * Hook into admin head to:
 * - prevent sellers from using Add Media and Add Form buttons.
 */
add_action( 'admin_head', function() {
	if ( bt_is_seller() || bt_is_subscriber_could_be_seller() ) {
		// don't show Add Media or Add Form buttons with editor
		remove_action( 'media_buttons', 'media_buttons' );
		remove_action( 'media_buttons', [ 'GFForms', 'add_form_button' ], 20 );
	}
} );


/**
 * Filter admin body class to add "seller" if the current user is a seller.
 *
 * Also do this if current user is a subscriber that could be a seller (e.g. cml/nma).
 */
add_filter( 'admin_body_class', function( $classes ) {
	if ( bt_is_seller() || bt_is_subscriber_could_be_seller() ) {
		$classes .= ' seller';
	}
	//error_log("ADMIN BODY CLASS: classes = $classes" );
	return $classes;
}, 9988, 1 );


/**
 * Hook into do_meta_boxes to, for sellers, remove meta boxes provided by plugins:
 * - The SEO Framework
 */
add_action( 'do_meta_boxes', function () {
	if ( bt_is_seller() || bt_is_subscriber_could_be_seller() ) {
		$screens = [
			'artist',
			'partner',
			'post',
			'product',
			'shop_order',
		];
		remove_meta_box( 'tsf-inpost-box', $screens, 'normal' );
	}
} );


/**
 * Hook into columns in wp-admin table for various post types to:
 * - Remove tinypng for sellers
 * - Attempt to remove The SEO Framework for sellers, but that column doesn't appear no matter the priority!
 */
//add_filter( 'manage_artist_posts_columns', 'bt_manage_columns', 9988 );
//add_filter( 'manage_product_posts_columns', 'bt_manage_columns', 9988 );
add_filter( 'manage_media_columns', 'bt_manage_columns', 9988 );
function bt_manage_columns( $columns ) {
	global $wp_query;
	$post_type = $wp_query->query['post_type'];

	//error_log( __METHOD__ . ' -- ' . $post_type . ' columns = ' . print_r( $columns, true ) );

	// remove tinypng column for sellers
	if ( $post_type == 'attachment' && ( bt_is_seller() || bt_is_subscriber_could_be_seller() ) ) {
		unset( $columns['tiny-compress-images'] );
	}

	return $columns;
}


/**
 * Suppress WordPress update notices.

&& wc_current_user_has_role( 'shop_manager' )
&& ! wc_current_user_has_role( 'administrator' )

 */
function bt_remove_upgrade_notices() {
	global $wp_version;
	return (object) [
		'last_checked'=> time(),
		'version_checked'=> $wp_version,
	];
}
if ( defined( 'BT_ENV' ) && BT_ENV != 'dev' ) {
	add_filter('pre_site_transient_update_core','bt_remove_upgrade_notices');
	add_filter('pre_site_transient_update_plugins','bt_remove_upgrade_notices');
	add_filter('pre_site_transient_update_themes','bt_remove_upgrade_notices');
}


add_filter( 'wc_authorize_net_cim_milestone_message', function( $message ) {
	if ( ! bt_can_admin_marketplace() ) {
		//error_log( __METHOD__ . ' -- omitting wc_authorize_net_cim_milestone_message since non-admin' );
		$message = '';
	}
	return $message;
} );


/**
 * Filter post counts so that:
 * - for sellers, we restrict to their posts
 *
 * Param info is from wp-includes/post.php
 *
 * @param object $counts An object containing the current post_type's post
 *                       counts by status.
 * @param string $type Post type.
 * @param string $perm The permission to determine if the posts are 'readable'
 *                       by the current user.
 *
 * @return object
 */
function bt_post_count_filter( $counts, $type, $perm ) {
	//error_log( 'COUNTS: counts = ' . print_r($counts,1) );
	//error_log( 'COUNTS: type = ' . $type . ' / perm = ' . $perm );

	// things to do for a seller in admin
	if (
		( bt_is_seller() || bt_is_subscriber_could_be_seller() )
		&& is_admin()
	) {
		//error_log('COUNTS: RUNNING on seller stuff.');

		/**
		 * START taken from wp-includes/post.php
		 * - we removed cache stuff
		 * - we made the query always use author id
		 */

		global $wpdb;

		if ( ! post_type_exists( $type ) ) {
			return new stdClass;
		}

		$query = "SELECT post_status, COUNT( * ) AS num_posts FROM {$wpdb->posts} WHERE post_type = %s";
		$query .= $wpdb->prepare(
			" AND post_author = %d ",
			get_current_user_id()
		);
		$query .= ' GROUP BY post_status';

		//error_log( 'COUNTS FILTER: query = ' . $query );

		$results = (array) $wpdb->get_results( $wpdb->prepare( $query, $type ), ARRAY_A );
		$counts  = array_fill_keys( get_post_stati(), 0 );

		foreach ( $results as $row ) {
			$counts[ $row['post_status'] ] = $row['num_posts'];
		}

		$counts = (object) $counts;

		/**
		 * END taken from wp-includes/post.php
		 */

	//} else {
		//error_log('COUNTS: NOT running on seller stuff.');
	}

	return $counts;
}
add_filter( 'wp_count_posts', 'bt_post_count_filter', 9988, 3 );


/**
 * Filter allowed uploaded file types to:
 * - prevent tiff uploading
 */
add_filter( 'upload_mimes', function( $mimes ) {
	//error_log( 'UPLOAD_MIMES: ' . print_r( $mimes, 1 ) );

	if ( bt_use_sellers() ) {
		/*
		$disallowed_mimes = [
			'tiff|tif',
			'psd',
			'zip',
		];

		foreach ( $disallowed_mimes as $mime ) {
			if ( isset( $mimes[$mime] ) ) {
				unset( $mimes[$mime] );
			}
		}
		*/

		$mimes = [
			'jpg|jpeg|jpe' => 'image/jpeg',
			'gif' => 'image/gif',
			'png' => 'image/png',
			'pdf' => 'application/pdf',
			'csv' => 'text/csv',
		];

    if ( bt_can_admin_marketplace() ) {
      // admins can upload mp4 videos
      $mimes['mp4'] = 'video/mp4';
    }
	}

	//error_log( 'UPLOAD_MIMES ADJ: ' . print_r( $mimes, 1 ) );

	return $mimes;
} );
